import hashlib

CHIFFRES = "0123456789"
MAJUSCULES = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
MINUSCULES = "abcdefghijklmnopqrztuvwxyz"
SYMBOLES = "&~#{([-_^@)]+=}$%*!:;.,?<>"
LONGUEUR_MAX_RECHERCHE = 5


def reconstitue_mdp(tab_index: list, alphabet) -> bytes:
    """Reconstitue le mot de passe à partir du tableau des index dans
    l'alphabet des symboles et renvoie le mot de passe sous forme d'un
    objet bytes.
    Ex : reconstitue_mdp([1,6],'ABCabc#-+') renverra 'B#'
         reconstitue_mdp([2,7,10],'012345abcdefghijkl') renverra '2be'
    """
    mdp = ""
    for index in tab_index:
        mdp += alphabet[index]
    return mdp.encode()

def combinaison_suivante(combinaison: list, nb_symboles_possibles: int) -> None:
    """Modifie le tableau combinaison en place pour passer à la combinaison
    suivante. La combinaison est un tableau d'entiers représentant l'index
    dans l'alphabet de chaque lettre du mot de passe.
    La fonction incrémente le dernier nombre du tableau et s'il dépasse
    nb_symboles_possibles, on le met à 0 et on incrémente l'avant-dernier_nombre
    si celui-ci dépasse aussi la limite, on le met à 0 et on incrémente le
    précédent et ainsi de suite.
    Exemples :
    combinaison_suivante([0,4,5],7) modifie le tableau qui devient [0,4,6]
    combinaison_suivante([2,3,8],9) modifie le tableau qui devient [2,4,0]
    combinaison_suivante([1,3,3],4) modifie le tableau qui devient [2,0,0]
    """
    position = len(combinaison) - 1  # Position de la modif à effectuer
    combinaison[position] = (combinaison[position] + 1) % nb_symboles_possibles
    # Si on a fait un tour (revenu à 0), on incrémente aussi le symbole précédent
    while combinaison[position] == 0 and position > 0:
        position -= 1    # On modifie le symbole précédent
        combinaison[position] = (combinaison[position] + 1) % nb_symboles_possibles


def retrouve_mdp(hash_mdp: bytes, longueur_max: int, alphabet: str,
                 algo_hash=hashlib.md5) -> str:
    """Tente de retrouver le mot de passe à partir de son hash et de l'alphabet
    utilisé avec une longueur maximale longueur_max.
    L'alphabet est une chaîne de caractère contenant tous les symboles
    utilisables. Ex : 'abcdefghijklmnopqrstuvwxyz0123456789' pour les minuscules
    et les chiffres.
    Renvoi le mot de passe sous forme d'une chaîne ou None s'il n'a pas été
    trouvé."""
    nb_symboles_possibles = len(alphabet)
    # TODO

def recherche_et_affiche_mdp(hash_mdp: bytes, longueur_max: int, alphabet: str,
                             algo_hash=hashlib.md5):
    """Tente de retrouver le mot de passe à partir de son hash et de l'alphabet
    utilisé avec une longueur maximale longueur_max.
    Cette fonction affiche le mot de passe s'il a été trouvé ou un message
    d'erreur dans le cas contraire et s'ppuie sur la fonction retrouve_mdp()."""
    mdp = retrouve_mdp(hash_mdp, LONGUEUR_MAX_RECHERCHE, alphabet, algo_hash=algo_hash)
    print()
    if mdp is None:
        print("Mot de passe non trouvé parmi l'alphabet", alphabet, "avec",
              LONGUEUR_MAX_RECHERCHE,"caractères maximum.")
    else:
        print("Mot de passe trouvé ! C'est :", mdp)    

##############
# QUESTION 1 #
##############
# On demande à l'utilisateur le mot de passe

# On le transforme en bytes

# Puis on calcule son hash

# Et on l'affiche



##############
# QUESTION 3 #
##############
alphabet = MINUSCULES
# On essaye maintenant toutes les combinaisons jusqu'à retouver le même hash
recherche_et_affiche_mdp(hash_mdp, LONGUEUR_MAX_RECHERCHE, alphabet)


##############
# QUESTION 4 #
##############
hash_a_trouver = "c67916bc13f15ed1689bc74bc8e4bcab"
hash_mdp = bytes.fromhex(hash_a_trouver)

# TODO


##############
# QUESTION 5 #
##############
hash_a_trouver = "8b02dfed70d50b12c13e8cc43cffd08ef9ca596697d51e4baaa3dd6a13473f6e"
hash_mdp = bytes.fromhex(hash_a_trouver)

# TODO
