import math
import random

PQ_MINI, PQ_MAXI = 100, 500
e = 65537
NOMBRE_A_ENVOYER = 452

def est_premier(n:int) -> bool:
    """Renvoi True si n est premier, False sinon."""
    for i in range(2, int(math.sqrt(n))+1):
        if n % i == 0:        # Si i est un multiple de n
            return False      # Alors il n'est pas premier
    return True               # Si on a tout essayé jusqu'à √n, il est premier

def premier_au_hasard(mini:int, maxi:int) -> int:
    """Renvoi un entier premier entre mini et maxi."""
    while True:
        n = random.randint(mini, maxi)
        if est_premier(n):
            return n

def premier_avec(n:int, mini=2) -> int:
    """Renvoi le premier nombre premier avec n en partant de mini"""
    prem = mini    # On démarre du minimum
    while n % prem == 0:  # Si prem divise n
        prem += 1         # Alors ils ne sont pas premiers et on teste le suivant
    return prem

def pgcd(a:int, b:int) -> int:
    """Renvoi le pgcd de a et b en utilisant l'algorithme d'euclide récursif."""
    if b == 0: return a
    return pgcd(b, a % b)

def euclide_etendu(a:int, b:int) -> tuple:
    """Renvoi un tuple (d, u, v) où d est le pgcd de a et b, et u, v sont des
    entiers relatifs avec au + bv = d."""
    if b == 0:
        return (a, 1, 0)
    else:
        (d, u, v) = euclide_etendu(b, a % b)
        return (d, v, u - (a//b)*v)

# Détermination de p et q, entiers pris au hasard
p = q = 0
while p == q:
    p = premier_au_hasard(PQ_MINI, PQ_MAXI)
    q = premier_au_hasard(PQ_MINI, PQ_MAXI)

n = p * q
p_moins_1_q_moins_1 = (p - 1) * (q - 1)

# Calcul de d à partir de e et (p-1)(q-1)
un, d, m = euclide_etendu(e, p_moins_1_q_moins_1)

# Vérification de la validité de d et recalcul en changeant e si nécessaire
mini = PQ_MINI         # Nombre minimal pour e
while d <= 1 or d*e + m*p_moins_1_q_moins_1 != 1:  # Si le d trouvé ne convient pas
    e = premier_avec(p_moins_1_q_moins_1, mini)    # On cherche un autre e
    un, d, m = euclide_etendu(e, p_moins_1_q_moins_1)  # et on calcule le nouveau d
    mini += 1          # Augmente la valeur minimale de e pour le prochain essai

# Affichage des résultats des calculs
print(f"p = {p}, q = {q}, n = {n}, e = {e}, d = {d}")
print(f"et m = {m} => On a bien {d}*{e} + {m}*{p_moins_1_q_moins_1} = " + \
      f"{d*e + m*p_moins_1_q_moins_1} = {(d*e + m*p_moins_1_q_moins_1) % (n)} [{n}]")

print(f"{'-'*40}\nClé publique (e, n) : ({e},{n})")
print(f"Clé privée (d, n) : ({d},{n})\n{'-'*40}")

print(f"Transmission du nombre {NOMBRE_A_ENVOYER} ...")
nb_envoye = (NOMBRE_A_ENVOYER ** e) % n
print(f"{NOMBRE_A_ENVOYER}^{e} [{n}] = {nb_envoye}")

print(f"Déchiffrement du nombre {nb_envoye} avec la clé secrète")
nb_dechiffre = (nb_envoye ** d) % n
print(f"{nb_envoye}^{d} [{n}] = {nb_dechiffre}")
if nb_dechiffre == NOMBRE_A_ENVOYER:
    print("La transmission s'est bien passée.")
else:
    print(f"Erreur dans la transmission : {NOMBRE_A_ENVOYER} a été envoyé et " \
          + f"c'est {nb_dechiffre} qui a été reçu après déchiffrement !")

