"""Ce programme permet de chiffrer un message en utilisant la clé publique
de quelqu'un (stockée dans un fichier) afin de lui envoyer ce message chiffré
avec le chiffrement RSA que lui seul pourra lire"""

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import hashes
import base64

NOM_FICHIER_CLE_PUBLIQUE = "Clé_publique.txt"
NOM_FICHIER_MESSAGE = "Message_chiffré.txt"

""" Le padding est l'ajout de données inutiles avant, après ou à l'intérieur
des données pour rendre plus difficile le déchiffrement.
Voir https://fr.abcdef.wiki/wiki/Padding_(cryptography)    """
PADDING = padding.OAEP(mgf=padding.MGF1(algorithm=hashes.SHA256()), algorithm=hashes.SHA256(),label=None)

def litFichierClePublique(nomFichier: str):
    """Récupère la clé publique se trouvant dans le fichier texte fournit en
   argument et la renvoie (ou None en cas d'erreur)."""
    cle_publique = None
    with open(nomFichier, "rb") as fichier_cle:
        cle_publique = serialization.load_pem_public_key(fichier_cle.read(),\
                                                      backend=default_backend())
    return cle_publique

def ecritureFichierMessage(message: bytes, nomFichier: str) -> None:
    """Ecrit le message sous forme de bytes dans un fichier texte en l'encodant
    en base64 avec des lignes de 80 caractères."""
    with open(nomFichier, "w") as fichier:
        # On converti le message en base64
        message = base64.b64encode(message).decode()
        # On découpe la chaine en lignes de 80 caractères
        lignes = [message[i:i+80] for i in range(0, len(message), 80)]
        for ligne in lignes:
            fichier.write(ligne + '\n')

# Lecture de la clé publique nécessaire au chiffrement à partir d'un fichier
nom_fichier_cle_publique = intput("Nom du fichier contenant la clé publique ?")
if nom_fichier_cle_publique == "":
    nom_fichier_cle_publique = NOM_FICHIER_CLE_PUBLIQUE
clePublique = litFichierClePublique(NOM_FICHIER_CLE_PUBLIQUE)

# Demande à l'utilisateur d'entrer le message et le passe en bytestring
message = input("Entrez le message à chiffrer : ").encode()

# Chiffrement du message en utilisant la clé publique
texteChiffre = clePublique.encrypt(message, PADDING)

# Et écriture du résultat du chiffrement dans un fichier
ecritureFichierMessage(texteChiffre, NOM_FICHIER_MESSAGE)


