"""Ce programme permet de déchiffrer un message crypté (stocké dans un fichier)
en utilisant sa clé secréte. Le message a du être chiffré avec la clé
publique correspondante."""

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives import hashes
import base64

NOM_FICHIER_CLE_PRIVE = "Clé_privée.txt"
NOM_FICHIER_MESSAGE = "Message_chiffré.txt"

# Algorithme de padding utilisé
""" Le padding est l'ajout de données inutiles avant, après ou à l'intérieur
des données pour rendre plus difficile le déchiffrement.
Voir https://fr.abcdef.wiki/wiki/Padding_(cryptography)
ou    https://wikimho.com/fr/q/crypto/3608                    """
PADDING = padding.OAEP(mgf=padding.MGF1(algorithm=hashes.SHA256()), algorithm=hashes.SHA256(),label=None)

def litFichierClePrive(nomFichier: str):
    """Récupère la clé privée se trouvant dans le fichier texte fournit en
       argument et la renvoie."""
    clePrive = None
    with open(nomFichier, "rb") as fichier_cle:
        clePrive = serialization.load_pem_private_key(fichier_cle.read(),\
                                                      password=None,\
                                                      backend=default_backend())
    return clePrive

def lectureFichierMessage(nomFichier: str) -> bytes:
    """Lit le fichier texte encodé en base 64 dont le nom est fourni en
       argument et renvoit un objet bytes correspondant aux octets contenus
       dans le fichier."""
    message = None
    with open(nomFichier, "r") as fichier:
        contenuFichier = fichier.readlines()
        message = base64.b64decode("".join(contenuFichier))
    return message

# Lecture de la clé secréte à partir du fichier
clePrive = litFichierClePrive(NOM_FICHIER_CLE_PRIVE)

# Lecture du fichier contenant le message à déchiffrer
messageChiffre = lectureFichierMessage(NOM_FICHIER_MESSAGE)

# Déchiffrement en utilisant la clé privée
messageEnClair = clePrive.decrypt(messageChiffre, PADDING)

# Affichage du messsage déchiffré
print("Le message déchiffré est :\n", messageEnClair.decode())
