"""Programme permettant de générer un couple clé privée/clé publique
et de les enregistrer dans deux fichiers séparés pour une utilisation
ultérieure."""

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization

NOM_FICHIER_CLE_PRIVE = "Clé_privée.txt"
NOM_FICHIER_CLE_PUBLIQUE = "Clé_publique.txt"


def ecritFichierCle(cleDecodee: str, nomFichier: str) -> None:
    """Ecrit dans un fichier texte la chaîne cleDecodee en la découpant en
       plusieurs lignes.
       Le fichier porte le nom donné en argument et sera écrasé sans
       confirmation s'il existe."""
    with open(nomFichier, "w") as fichier_cle:
        for ligne in cleDecodee.splitlines():
            fichier_cle.write(ligne + '\n')
            
def ecritFichierClePrive(cle, nomFichier: str) -> None:
    """Ecrit dans un fichier texte la clé privée au format .PEM
    La clé est fournie au format interne de cryptography et le fichier, s'il
    existe est écrasé."""
    cleDecodee = cle.private_bytes(encoding=serialization.Encoding.PEM, \
                             format=serialization.PrivateFormat.PKCS8,
                             encryption_algorithm=serialization.NoEncryption()).decode()
    ecritFichierCle(cleDecodee, nomFichier)

def ecritFichierClePublique(cle, nomFichier: str) -> None:
    """Ecrit dans un fichier texte la clé puplique au format .PEM
    La clé est fournie au format interne de cryptography et le fichier, s'il
    existe est écrasé."""
    cleDecodee = cle.public_bytes(encoding=serialization.Encoding.PEM, \
                             format=serialization.PublicFormat.SubjectPublicKeyInfo).decode()
    ecritFichierCle(cleDecodee, nomFichier)


# Création d'un nouveau couple clé privée/clé publique
clePrive = rsa.generate_private_key(public_exponent=65537,\
                                       key_size=4096, backend=default_backend())

# Ecriture de la clé privée dans un fichier
ecritFichierClePrive(clePrive, NOM_FICHIER_CLE_PRIVE)

# Extraction de la clé publique correspondante
clePublique = clePrive.public_key()

# Ecriture de la clé publique dans un autre fichier
ecritFichierClePublique(clePublique, NOM_FICHIER_CLE_PUBLIQUE)
